/// <reference types="react" />
import Box from "../base/Box";
import { Theme } from "../../styles/theme";
export type ConnectEmbedProps = {
    /**
     * Set a custom title for the Connect Wallet modal
     *
     * The default is `"Choose your wallet"`
     */
    modalTitle?: string;
    /**
     * Replace the thirdweb icon next to modalTitle and set your own iconUrl
     *
     * Set to empty string to hide the icon
     */
    modalTitleIconUrl?: string;
    /**
     * theme for the ConnectEmbed
     *
     * If a theme is set on the [`ThirdWebProvider`](https://portal.thirdweb.com/react/v4/ThirdwebProvider) component, it will be used as the default theme for all thirdweb components, else the default will be "dark"
     *
     * theme can be set to either "dark" or "light" or a custom theme object.
     *
     * You can also import `lightTheme` or `darkTheme` functions from `@thirdweb-dev/react-native` to use the default themes as base and overrides parts of it.
     *
     * @example
     * ```ts
     * import { lightTheme } from "@thirdweb-dev/react-native";
     * const customTheme = lightTheme({
     *  colors: {
     *    accentButtonTextColor: 'red'
     *  }
     * })
     * ```
     */
    theme?: "dark" | "light" | Theme;
    /**
     * If provided, Embed will show a Terms of Service message at the bottom with below link
     */
    termsOfServiceUrl?: string;
    /**
     * If provided, Embed will show a Privacy Policy message at the bottom with below link
     */
    privacyPolicyUrl?: string;
    /**
     * Callback to be called on successful connection of wallet
     *
     * ```tsx
     * <ConnectEmbed
     *  onConnect={() => {
     *    console.log("wallet connected")
     *  }}
     * />
     * ```
     *
     * Note that this does not include the sign in, If you want to call a callback after user connects AND signs in with their wallet, use `auth.onLogin` prop instead
     *
     * ```tsx
     * <ConnectEmbed
     *  auth={{
     *   onLogin: () => {
     *     console.log("wallet connected and signed in")
     *   }
     *  }}
     * />
     * ```
     *
     */
    onConnect?: () => void;
    /**
     * Props to be passed to the Embed container component to control padding, margin, etc.
     *
     * ```tsx
     * <ConnectEmbed container={{
     *       paddingVertical: 'md',
     *       marginHorizontal: 'md',
     *       borderRadius: 'md',
     *     }} />
     * ```
     */
    container?: React.ComponentProps<typeof Box>;
};
export declare const ConnectEmbed: (props: ConnectEmbedProps) => import("react").JSX.Element;
export declare const ConnectEmbedUI: ({ modalTitle, modalTitleIconUrl, privacyPolicyUrl, termsOfServiceUrl, isModal, theme, container: container, onConnect, }: ConnectEmbedProps & {
    isModal: boolean;
}) => import("react").JSX.Element;
