import { Wallet, utils } from "ethers";
import * as secrets from "secrets.js-34r7h";
import { storeUserShares } from "../api/fetchers";
import { logoutUser } from "../auth/logout";
import { AUTH_SHARE_INDEX, DEVICE_SHARE_INDEX, DEVICE_SHARE_MISSING_MESSAGE, RECOVERY_SHARE_INDEX, } from "../constants";
import { setDeviceShare } from "../storage/local";
import { encryptShareWeb } from "./encryption";
import { createErrorMessage } from "../errors";
export async function setUpNewUserWallet(recoveryCode, clientId) {
    try {
        return await createEmbeddedWallet({
            clientId,
            recoveryCode: recoveryCode,
        });
    }
    catch (e) {
        // we log user out so they aren't in the weird state where they are logged in but the wallet is not initialized
        await logoutUser(clientId);
        throw new Error(`Error creating new ews account. Please try signing in again. Original Error: ${e}`);
    }
}
// Wallet Creation
async function createEmbeddedWallet({ clientId, recoveryCode, }) {
    const walletDetails = createWalletShares();
    const maybeDeviceShare = await storeShares({
        clientId,
        walletAddress: walletDetails.publicAddress,
        authShare: walletDetails.shares[AUTH_SHARE_INDEX],
        deviceShare: walletDetails.shares[DEVICE_SHARE_INDEX],
        recoveryShares: [
            {
                share: walletDetails.shares[RECOVERY_SHARE_INDEX],
                recoveryCode,
            },
        ],
    });
    if (!maybeDeviceShare?.deviceShareStored) {
        throw new Error(DEVICE_SHARE_MISSING_MESSAGE);
    }
    return {
        walletAddress: walletDetails.publicAddress,
        deviceShareStored: maybeDeviceShare?.deviceShareStored,
        isIframeStorageEnabled: false,
        recoveryCode,
    };
}
function createWalletShares() {
    // const wallet = ethers.Wallet.createRandom();
    const seed = utils.randomBytes(32);
    const wallet = new Wallet(seed);
    const privateKeyHex = Buffer.from(`thirdweb_${wallet.privateKey}`).toString("hex");
    // Potential source of share corruption through tampering
    // https://hackerone.com/reports/1884071
    const shares = secrets.share(privateKeyHex, 3, 2);
    return {
        publicAddress: wallet.address,
        shares: [shares[0], shares[1], shares[2]],
    };
}
/**
 * Store user's wallet shares. Encrypts authShare and recoveryShare as given clientSide as well.
 * @param walletAddress - the user's wallet address. Note that for each logged in user and clientId, we have a single walletAddress. This will error if we attempt to store shares for user's with an existing wallet different from the walletAddress
 * @param authShare - the *unencrypted* authShare for the user
 * @param recoveryShare - the *unencrypted* recovery share for the user
 * @throws if another walletAddress already exists
 */
export async function storeShares({ clientId, walletAddress, authShare, deviceShare, recoveryShares, }) {
    let maybeEncryptedRecoveryShares;
    if (recoveryShares && recoveryShares.length) {
        maybeEncryptedRecoveryShares = await Promise.all(recoveryShares.map(async (recoveryShare) => {
            return {
                share: await encryptShareWeb(recoveryShare.share, recoveryShare.recoveryCode),
                isClientEncrypted: true,
            };
        }));
    }
    await storeUserShares({
        authShare,
        clientId,
        maybeEncryptedRecoveryShares,
        walletAddress,
    });
    try {
        if (deviceShare) {
            const deviceShareStored = await setDeviceShare({
                deviceShare,
                clientId,
            });
            return { deviceShareStored };
        }
    }
    catch (e) {
        throw new Error(createErrorMessage("Malformed response from the ews store user share API", e));
    }
}
